"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sha1 = require("js-sha1");
var ProjectConfig_1 = require("./ProjectConfig");
var semver = require("semver");
var util_1 = require('react-native-util');
/** Object for variation evaluation */
var User = /** @class */ (function () {
    function User(identifier, email, country, custom) {
        if (email === void 0) { email = null; }
        if (country === void 0) { country = null; }
        if (custom === void 0) { custom = {}; }
        /** Optional dictionary for custom attributes of the User for advanced targeting rule definitions. e.g. User role, Subscription type */
        this.custom = {};
        this.identifier = identifier;
        this.email = email;
        this.country = country;
        this.custom = custom;
    }
    return User;
}());
exports.User = User;
var RolloutEvaluator = /** @class */ (function () {
    function RolloutEvaluator(logger) {
        this.logger = logger;
    }
    RolloutEvaluator.prototype.Evaluate = function (config, key, defaultValue, user) {
        if (!config || !config.ConfigJSON) {
            this.logger.error("JSONConfig is not present. Returning default value: '" + defaultValue + "'.");
            return defaultValue;
        }
        if (!config.ConfigJSON[key]) {
            var s = "Evaluating getValue('" + key + "') failed. Returning default value: '" + defaultValue + "'.";
            s += " Here are the available keys: {" + Object.keys(config.ConfigJSON).join() + "}.";
            this.logger.error(s);
            return defaultValue;
        }
        var eLog = new EvaluateLogger();
        eLog.User = user;
        eLog.KeyName = key;
        eLog.ReturnValue = defaultValue;
        var result = new EvaluateResult();
        result.EvaluateLog = eLog;
        if (user) {
            result = this.EvaluateRules(config.ConfigJSON[key][ProjectConfig_1.Setting.RolloutRules], user, eLog);
            if (result.Value == null) {
                result.Value = result.EvaluateLog.ReturnValue = this.EvaluateVariations(config.ConfigJSON[key][ProjectConfig_1.Setting.RolloutPercentageItems], key, user);
                result.EvaluateLog.OpAppendLine("Evaluate % option => " + (result.Value == null ? "user not targeted" : "user targeted"));
            }
        }
        else {
            if ((config.ConfigJSON[key][ProjectConfig_1.Setting.RolloutRules] && config.ConfigJSON[key][ProjectConfig_1.Setting.RolloutRules].length > 0) ||
                (config.ConfigJSON[key][ProjectConfig_1.Setting.RolloutPercentageItems] && config.ConfigJSON[key][ProjectConfig_1.Setting.RolloutPercentageItems].length > 0)) {
                var s = "Evaluating getValue('" + key + "'). ";
                s += "UserObject missing! You should pass a UserObject to getValue(), in order to make targeting work properly. ";
                s += "Read more: https://configcat.com/docs/advanced/user-object";
                this.logger.warn(s);
            }
        }
        if (result.Value == null) {
            result.EvaluateLog.ReturnValue = defaultValue;
        }
        this.logger.info(result.EvaluateLog.GetLog());
        return result.Value == null ? config.ConfigJSON[key][ProjectConfig_1.Setting.Value] : result.Value;
    };
    RolloutEvaluator.prototype.EvaluateRules = function (rolloutRules, user, eLog) {
        var result = new EvaluateResult();
        result.Value = null;
        if (rolloutRules && rolloutRules.length > 0) {
            var _loop_1 = function (i) {
                var rule = rolloutRules[i];
                var comparisonAttribute = this_1.GetUserAttribute(user, rule[ProjectConfig_1.RolloutRules.ComparisonAttribute]);
                if (!comparisonAttribute) {
                    return "continue";
                }
                var comparator = rule[ProjectConfig_1.RolloutRules.Comparator];
                var comparisonValue = rule[ProjectConfig_1.RolloutRules.ComparisonValue];
                var log = "Evaluate rule: '" + comparisonAttribute + "' " + this_1.RuleToString(comparator) + " '" + comparisonValue + "' => ";
                switch (comparator) {
                    case 0: // in
                        var cvs = comparisonValue.split(",");
                        for (var ci = 0; ci < cvs.length; ci++) {
                            if (cvs[ci].trim() === comparisonAttribute) {
                                log += "MATCH";
                                eLog.OpAppendLine(log);
                                result.Value = eLog.ReturnValue = rule[ProjectConfig_1.RolloutRules.Value];
                                result.EvaluateLog = eLog;
                                return { value: result };
                            }
                        }
                        log += "no match";
                        break;
                    case 1: // notIn
                        if (!comparisonValue.split(",").some(function (e) {
                            if (e.trim() === comparisonAttribute) {
                                return true;
                            }
                            return false;
                        })) {
                            log += "MATCH";
                            eLog.OpAppendLine(log);
                            result.Value = eLog.ReturnValue = rule[ProjectConfig_1.RolloutRules.Value];
                            result.EvaluateLog = eLog;
                            return { value: result };
                        }
                        log += "no match";
                        break;
                    case 2: // contains
                        if (comparisonAttribute.search(comparisonValue) !== -1) {
                            log += "MATCH";
                            eLog.OpAppendLine(log);
                            result.Value = eLog.ReturnValue = rule[ProjectConfig_1.RolloutRules.Value];
                            result.EvaluateLog = eLog;
                            return { value: result };
                        }
                        log += "no match";
                        break;
                    case 3: // not contains
                        if (comparisonAttribute.search(comparisonValue) === -1) {
                            log += "MATCH";
                            eLog.OpAppendLine(log);
                            result.Value = eLog.ReturnValue = rule[ProjectConfig_1.RolloutRules.Value];
                            result.EvaluateLog = eLog;
                            return { value: result };
                        }
                        log += "no match";
                        break;
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                    case 8:
                    case 9:
                        if (this_1.EvaluateSemver(comparisonAttribute, comparisonValue, comparator)) {
                            log += "MATCH";
                            eLog.OpAppendLine(log);
                            result.Value = eLog.ReturnValue = rule[ProjectConfig_1.RolloutRules.Value];
                            result.EvaluateLog = eLog;
                            return { value: result };
                        }
                        log += "no match";
                        break;
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                        if (this_1.EvaluateNumber(comparisonAttribute, comparisonValue, comparator)) {
                            log += "MATCH";
                            eLog.OpAppendLine(log);
                            result.Value = eLog.ReturnValue = rule[ProjectConfig_1.RolloutRules.Value];
                            result.EvaluateLog = eLog;
                            return { value: result };
                        }
                        log += "no match";
                        break;
                    default:
                        break;
                }
                eLog.OpAppendLine(log);
            };
            var this_1 = this;
            for (var i = 0; i < rolloutRules.length; i++) {
                var state_1 = _loop_1(i);
                if (typeof state_1 === "object")
                    return state_1.value;
            }
        }
        result.EvaluateLog = eLog;
        return result;
    };
    RolloutEvaluator.prototype.EvaluateVariations = function (rolloutPercentageItems, key, User) {
        if (rolloutPercentageItems && rolloutPercentageItems.length > 0) {
            var hashCandidate = key + User.identifier;
            var hashValue = sha1(hashCandidate).substring(0, 7);
            var hashScale = parseInt(hashValue, 16) % 100;
            var bucket = 0;
            for (var i = 0; i < rolloutPercentageItems.length; i++) {
                var variation = rolloutPercentageItems[i];
                bucket += +variation[ProjectConfig_1.RolloutPercentageItems.Percentage];
                if (hashScale < bucket) {
                    return variation[ProjectConfig_1.RolloutPercentageItems.Value];
                }
            }
        }
        return null;
    };
    RolloutEvaluator.prototype.EvaluateNumber = function (v1, v2, comparator) {
        var n1, n2;
        if (v1 && !Number.isNaN(Number.parseFloat(v1.replace(',', '.')))) {
            n1 = Number.parseFloat(v1.replace(',', '.'));
        }
        else {
            return false;
        }
        if (v2 && !Number.isNaN(Number.parseFloat(v2.replace(',', '.')))) {
            n2 = Number.parseFloat(v2.replace(',', '.'));
        }
        else {
            return false;
        }
        switch (comparator) {
            case 10:
                return n1 == n2;
            case 11:
                return n1 != n2;
            case 12:
                return n1 < n2;
            case 13:
                return n1 <= n2;
            case 14:
                return n1 > n2;
            case 15:
                return n1 >= n2;
            default:
                break;
        }
        return false;
    };
    RolloutEvaluator.prototype.EvaluateSemver = function (v1, v2, comparator) {
        if (semver.valid(v1) == null || util_1.isUndefined(v2)) {
            return false;
        }
        v2 = v2.trim();
        switch (comparator) {
            case 4:
                // in
                var sv = v2.split(",");
                var found = false;
                for (var ci = 0; ci < sv.length; ci++) {
                    if (!sv[ci] || util_1.isUndefined(sv[ci]) || sv[ci].trim() === "") {
                        continue;
                    }
                    if (semver.valid(sv[ci].trim()) == null) {
                        return false;
                    }
                    if (!found) {
                        found = semver.eq(v1, sv[ci].trim(), true);
                    }
                }
                return found;
            case 5:
                // not in
                return !v2.split(",").some(function (e) {
                    if (!e || util_1.isUndefined(e) || e.trim() === "") {
                        return false;
                    }
                    e = semver.valid(e.trim());
                    if (e == null) {
                        return false;
                    }
                    return semver.eq(v1, e);
                });
            case 6:
                if (semver.valid(v2) == null) {
                    return false;
                }
                return semver.lt(v1, v2);
            case 7:
                if (semver.valid(v2) == null) {
                    return false;
                }
                return semver.lte(v1, v2);
            case 8:
                if (semver.valid(v2) == null) {
                    return false;
                }
                return semver.gt(v1, v2);
            case 9:
                if (semver.valid(v2) == null) {
                    return false;
                }
                return semver.gte(v1, v2);
            default:
                break;
        }
        return false;
    };
    RolloutEvaluator.prototype.GetUserAttribute = function (User, attribute) {
        switch (attribute) {
            case "Identifier":
                return User.identifier;
            case "Email":
                return User.email;
            case "Country":
                return User.country;
            default:
                return (User.custom || {})[attribute];
        }
    };
    RolloutEvaluator.prototype.RuleToString = function (rule) {
        switch (rule) {
            case 0:
            case 4:
                return "IN";
            case 1:
            case 5:
                return "NOT IN";
            case 2:
                return "CONTAINS";
            case 3:
                return "NOT CONTAINS";
            case 6:
            case 12:
                return "<";
            case 7:
            case 13:
                return "<=";
            case 8:
            case 14:
                return ">";
            case 9:
            case 15:
                return ">=";
            case 10:
                return "=";
            case 11:
                return "!=";
            default:
                return rule;
        }
    };
    return RolloutEvaluator;
}());
exports.RolloutEvaluator = RolloutEvaluator;
var EvaluateResult = /** @class */ (function () {
    function EvaluateResult() {
    }
    return EvaluateResult;
}());
var EvaluateLogger = /** @class */ (function () {
    function EvaluateLogger() {
        this.Operations = "";
    }
    EvaluateLogger.prototype.OpAppendLine = function (s) {
        this.Operations += " " + s + "\n";
    };
    EvaluateLogger.prototype.GetLog = function () {
        return "Evaluate '" + this.KeyName + "'"
            + "\n User : " + JSON.stringify(this.User)
            + "\n" + this.Operations
            + " Returning value : " + this.ReturnValue;
    };
    return EvaluateLogger;
}());
